@@ -13,6 +13,7 @@ from piptools.scripts.compile import cli
 from .constants import MINIMAL_WHEELS_PATH, PACKAGES_PATH
 
 is_pypy = "__pypy__" in sys.builtin_module_names
+is_windows = sys.platform == "win32"
 
 
 @pytest.fixture(autouse=True)
@@ -348,6 +349,9 @@ def test_emit_index_url_option(runner, option, expected_output):
 
 
 @pytest.mark.network
+@pytest.mark.xfail(
+    is_pypy and is_windows, reason="https://github.com/jazzband/pip-tools/issues/1148"
+)
 def test_realistic_complex_sub_dependencies(runner):
     wheels_dir = "wheels"
 
@@ -1723,6 +1727,38 @@ def test_duplicate_reqs_combined(
     assert "test-package-1==0.1" in out.stderr
 
 
+def test_combine_extras(pip_conf, runner, make_package):
+    """
+    Ensure that multiple declarations of a dependency that specify different
+    extras produces a requirement for that package with the union of the extras
+    """
+    package_with_extras = make_package(
+        "package_with_extras",
+        extras_require={
+            "extra1": ["small-fake-a==0.1"],
+            "extra2": ["small-fake-b==0.1"],
+        },
+    )
+
+    with open("requirements.in", "w") as req_in:
+        req_in.writelines(
+            [
+                "-r ./requirements-second.in\n",
+                f"{package_with_extras}[extra1]",
+            ]
+        )
+
+    with open("requirements-second.in", "w") as req_sec_in:
+        req_sec_in.write(f"{package_with_extras}[extra2]")
+
+    out = runner.invoke(cli, ["-n"])
+
+    assert out.exit_code == 0
+    assert "package-with-extras" in out.stderr
+    assert "small-fake-a==" in out.stderr
+    assert "small-fake-b==" in out.stderr
+
+
 @pytest.mark.parametrize(
     ("pkg2_install_requires", "req_in_content", "out_expected_content"),
     (
@@ -1929,9 +1965,16 @@ def test_one_extra(fake_dists, runner, make_module, fname, content):
 
 
 @pytest.mark.network
+@pytest.mark.parametrize(
+    "extra_opts",
+    (
+        pytest.param(("--extra", "dev", "--extra", "test"), id="singular"),
+        pytest.param(("--extra", "dev,test"), id="comma-separated"),
+    ),
+)
 @pytest.mark.parametrize(("fname", "content"), METADATA_TEST_CASES)
 @pytest.mark.xfail(is_pypy, reason="https://github.com/jazzband/pip-tools/issues/1375")
-def test_multiple_extras(fake_dists, runner, make_module, fname, content):
+def test_multiple_extras(fake_dists, runner, make_module, fname, content, extra_opts):
     """
     Test passing multiple `--extra` params.
     """
@@ -1940,10 +1983,7 @@ def test_multiple_extras(fake_dists, runner, make_module, fname, content):
         cli,
         [
             "-n",
-            "--extra",
-            "dev",
-            "--extra",
-            "test",
+            *extra_opts,
             "--find-links",
             fake_dists,
             meta_path,
